@@ -26,10 +26,7 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 #include <net/if.h>
-#include <linux/atm.h>
 #include <linux/atmdev.h>
-#include <linux/atmarp.h>
-#include <linux/atmclip.h>
 #include "io.h"
 #include "atmd.h"
 #include "atm.h"
