@@ -31,7 +31,7 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <stdint.h> /* includes linux/atm.h */
+#include <atm.h>
 #include <linux/atmlec.h>
 
 /* Include General Headers. */
@@ -75,6 +75,8 @@ typedef struct {
   ARP_XMT_CALLBACK     arp_xmt_callback;
   FLUSH_XMT_CALLBACK   flush_xmt_callback;
   SVC_SETUP_CALLBACK   svc_setup_callback;
+  void                 (*associate_req_callback) (HANDLE lc_elan_handle,
+                                                  int sizeoftlvs); /* LANE2 */
   LA_ELAN_LIST         elan_list;
 } LA_CONTEXT;
 
@@ -90,7 +92,10 @@ kernel_arp_xmt_callback(struct atmlec_msg *mesg)
   LA_ELAN_CONTEXT *p_elan;
   ESI *to_pass;
 
-  to_pass = (ESI*)mesg->content.normal.mac_addr;
+  if (mesg->content.normal.targetless_le_arp)
+    to_pass = NULL;
+  else
+    to_pass = (ESI*)mesg->content.normal.mac_addr;
 
   utl_list_traverse(lec_arp_context.elan_list, p_elan) {
     lec_arp_context.arp_xmt_callback(p_elan->lc_elan_handle,
@@ -108,7 +113,7 @@ kernel_flush_xmt(unsigned char *to_pass)
     lec_arp_context.flush_xmt_callback(p_elan->lc_elan_handle, to_pass,&dumb);
     EVENT(EM_DEBUG,("Setting flush transaction id to %lx\n",dumb));
     kernel_sendmsg(l_flush_tran_id, NULL, to_pass,
-		   NULL, dumb, NULL, 0);
+		   NULL, dumb, NULL, 0, 0, 0);
   }
 }
 
@@ -125,21 +130,35 @@ kernel_svc_setup_xmt_callback(struct atmlec_msg *mesg)
   }
 }
 
+/* LANE2: LE_ASSOCIATE.request */
+static void
+kernel_associate_req_callback(struct atmlec_msg *mesg)
+{
+  LA_ELAN_CONTEXT *p_elan;
+
+  utl_list_traverse(lec_arp_context.elan_list, p_elan) {
+    lec_arp_context.associate_req_callback(p_elan->lc_elan_handle, mesg->sizeoftlvs);
+  }
+}
+
 STATUS 
 la_create (HANDLE lc_handle, ARP_XMT_CALLBACK arp_xmt_callback,
 	   FLUSH_XMT_CALLBACK flush_xmt_callback,
 	   SVC_SETUP_CALLBACK svc_setup_callback,
+           void (*associate_req_callback)(HANDLE lc_elan_handle, int sizeoftlvs), /* LANE2 */
 	   HANDLE *p_la_handle)
 {
   lec_arp_context.lc_handle = lc_handle;
   lec_arp_context.arp_xmt_callback    = arp_xmt_callback;
   lec_arp_context.flush_xmt_callback  = flush_xmt_callback;
   lec_arp_context.svc_setup_callback  = svc_setup_callback;
+  lec_arp_context.associate_req_callback = associate_req_callback; /* LANE2 */
   
   utl_list_init(lec_arp_context.elan_list);
 
   kernel_register_callback(l_arp_xmt, kernel_arp_xmt_callback);
   kernel_register_callback(l_svc_setup, kernel_svc_setup_xmt_callback);
+  kernel_register_callback(l_associate_req, kernel_associate_req_callback);
 
   *p_la_handle = (HANDLE)&lec_arp_context;
 
@@ -212,7 +231,8 @@ la_config(HANDLE la_elan_handle, LAN_TYPE lan_type,
 	  UINT16 max_unknown_frame_time, UINT32 vcc_timeout,
 	  UINT16 max_retry_count, UINT32 aging_time,
 	  UINT16 forward_delay_time, UINT16 le_arp_response_time,
-	  UINT16 flush_timeout, UINT16 path_switching_delay)
+	  UINT16 flush_timeout, UINT16 path_switching_delay,
+          unsigned int lane_version_number)
 {
   LA_ELAN_CONTEXT   *p_elan;
   struct atmlec_config_msg conf_mesg;
@@ -228,7 +248,8 @@ la_config(HANDLE la_elan_handle, LAN_TYPE lan_type,
   conf_mesg.arp_response_time = le_arp_response_time;
   conf_mesg.flush_timeout = flush_timeout;
   conf_mesg.path_switching_delay = path_switching_delay;
-  kernel_sendmsg(l_config, NULL, NULL, &conf_mesg, 0, NULL, 0);
+  conf_mesg.lane_version = lane_version_number; /* LANE2 */
+  kernel_sendmsg(l_config, NULL, NULL, &conf_mesg, 0, NULL, 0, 0, 0);
 }
 
 STATUS 
@@ -237,7 +258,7 @@ la_addr_delete(HANDLE la_elan_handle,
 	       BOOLEAN permanent)
 {
   kernel_sendmsg(l_addr_delete, NULL, atm_addr, NULL,
-		  permanent==TRUE?1:0, NULL, 0);
+		  permanent==TRUE?1:0, NULL, 0, 0, 0);
   return STATUS_K_SUCCESS;
 }
 
@@ -246,18 +267,20 @@ la_addr_delete(HANDLE la_elan_handle,
  *
  */
 void 
-la_arp_update(HANDLE    la_elan_handle,
-	      ESI       esi,
-	      ADDR_ATM  addr_atm,
-	      BOOLEAN   remote_flag,
-              char      *tlv_list,
-              int       sizeoftlvs)
+la_arp_update(HANDLE       la_elan_handle,
+	      ESI          esi,
+	      ADDR_ATM     addr_atm,
+	      BOOLEAN      remote_flag,
+              char         *tlv_list,
+              int          sizeoftlvs,
+              int          targetless_le_arp,
+              int          no_source_le_narp)
 {
   EVENT(EM_DEBUG,("ARP update for %2.2x%2.2x%2.2x%2.2x%2.2x%2.2x\n",
 		  esi[0],esi[1],esi[2],esi[3],esi[4],esi[5]));
   kernel_sendmsg(l_arp_update, (unsigned char *)esi,
 		 (unsigned char *)&addr_atm, NULL, remote_flag==TRUE?1:0,
-                 tlv_list, sizeoftlvs);
+                 tlv_list, sizeoftlvs, targetless_le_arp, no_source_le_narp);
 }
 
 void 
@@ -266,7 +289,7 @@ la_flush_complete(HANDLE la_elan_handle,
 {
   EVENT(EM_DEBUG,("Flush complete for tran_id %lx\n", tran_id));
   kernel_sendmsg(l_flush_complete, NULL,
-		 NULL, NULL, tran_id, NULL, 0);
+		 NULL, NULL, tran_id, NULL, 0, 0, 0);
 }
 
 void 
@@ -275,7 +298,7 @@ la_topology_change_set(HANDLE  la_elan_handle,
 {
   kernel_sendmsg(l_topology_change, NULL, NULL, NULL,
 		 topology_change_flag==TRUE?1:0,
-                 NULL, 0);
+                 NULL, 0, 0, 0);
 }
 
 
