@@ -1,6 +1,6 @@
 /* ans.c - Interface for text2atm and atm2text to ANS */
 
-/* Written 1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1996,1998 by Werner Almesberger, EPFL-LRC/ICA */
 
 
 /*
@@ -20,7 +20,6 @@
 #include <arpa/nameser.h>
 #include <netdb.h>
 #include <resolv.h>
-#include <linux/atm.h>
 
 #include "atm.h"
 #include "atmres.h"
