@@ -123,13 +123,18 @@ static void usage(const char *name)
       name);
     fprintf(stderr,"%s [ -b ] [ -d ] [ -i itf ] -s <host> <line> "
       "[ <rmtport> ]\n",name);
+    fprintf(stderr,"%s -p [ [ -i ] itf ]\n",name);
+    fprintf(stderr,"%s -r [ [ -i ] itf ]\n",name);
+    fprintf(stderr,"\n  -b  background\n  -d  debug\n  -i  interface\n");
+    fprintf(stderr,"  -l  listen\n  -c  connect\n  -s  connect to switch\n");
+    fprintf(stderr,"  -p  create persistent\n  -r  remove persistent\n");
     exit(1);
 }
 
 
 int main(int argc,char **argv)
 {
-    enum { m_undefined,m_listen,m_connect,m_switch } mode;
+    enum { m_undefined,m_listen,m_connect,m_switch,m_create,m_remove } mode;
     struct sockaddr_in addr;
     const char *name;
     char *end;
@@ -143,7 +148,8 @@ int main(int argc,char **argv)
     background = 0;
     host = line = NULL; /* for gcc */
     itf = -1;
-    while ((c = getopt(argc,argv,"bdi:lc:s:")) != EOF)
+    port = 0; /* for gcc */
+    while ((c = getopt(argc,argv,"bdi:lc:prs:")) != EOF)
 	switch (c) {
 	    case 'b':
 		background = 1;
@@ -170,6 +176,14 @@ int main(int argc,char **argv)
 		mode = m_switch;
 		host = optarg;
 		break;
+	    case 'p':
+		if (mode != m_undefined) usage(name);
+		mode = m_create;
+		break;
+	    case 'r':
+		if (mode != m_undefined) usage(name);
+		mode = m_remove;
+		break;
 	    default:
 		usage(name);
         }
@@ -185,9 +199,30 @@ int main(int argc,char **argv)
 	    if (argc > optind+1) usage(name);
 	    port = argc == optind+1 ? atoi(argv[optind]) : -1;
 	    break;
+	case m_remove:
+	    /* fall through */
+	case m_create:
+	    if (background || debug) usage(name);
+	    if (argc != optind && itf != -1) usage(name);
+	    if (argc != optind+1) usage(name);
+	    itf = strtoul(argv[optind],&end,10);
+	    if (*end) usage(name);
+	    break;
 	default:
 	    abort();
     }
+    if ((s_krn = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0) {
+	perror("socket");
+	return 1;
+    }
+    if (mode == m_create || mode == m_remove) {
+	if (mode == m_remove && itf == -1) itf = 0;
+	if (ioctl(s_krn,mode == m_create ? ATMTCP_CREATE : ATMTCP_REMOVE,itf)
+	  >= 0) return 0;
+	perror(mode == m_create ?
+	  "ioctl ATMTCP_CREATE" : "ioctl ATMTCP_REMOVE");
+	return 1;
+    }
     if ((s_tcp = socket(PF_INET,SOCK_STREAM,0)) < 0) {
 	perror("socket");
 	return 1;
@@ -199,10 +234,6 @@ int main(int argc,char **argv)
      * Create atmtcp interface before setting up the TCP connection in order
      * to make the assignment of interface numbers a bit more predictable.
      */
-    if ((s_krn = socket(PF_ATMSVC,SOCK_DGRAM,0)) < 0) {
-	perror("socket");
-	return 1;
-    }
     if ((itf = ioctl(s_krn,SIOCSIFATMTCP,itf)) < 0) {
 	perror("ioctl SIOCSIFATMTCP");
 	return 1;
