@@ -78,7 +78,7 @@ static int accept_conn(int slot);
 static int add_shortcut(int slot, int type);
 static int check_connection(int slot);
 static int complete_connection(int slot);
-static int create_shortcut(char *atm_addr);
+static int create_shortcut(char *atm_addr,struct atm_qos qos);
 static void wait_for_mps_ctrl_addr(void);
 static int connect_to_MPS(void); 
 
@@ -581,18 +581,18 @@ static int complete_connection(int slot)
 /*
  * Called if kernel wants us to create a shortcut
  */
-void create_ingress_svc(uint32_t ipaddr, char *atm_addr)
+void create_ingress_svc(uint32_t ipaddr, char *atm_addr, struct atm_qos qos)
 {
 	int i, new_socket;
-
-        new_socket = create_shortcut(atm_addr);
+	
+        new_socket = create_shortcut(atm_addr,qos);
         if (new_socket < 0) {
                 printf("mpcd: io.c: create_ingress_svc: create_shortcut failed\n");
                 return;
 	}
         
         for (i = first_empty; i < OPEN_MAX; i++) {
-                if (fds[i].fd >= 0)                /* slot in use ? */
+	        if (fds[i].fd >= 0)                /* slot in use ? */
                         continue;
                 fds[i].fd     = new_socket;
                 fds[i].events = POLLIN | POLLOUT;
@@ -627,7 +627,7 @@ void create_ingress_svc(uint32_t ipaddr, char *atm_addr)
  * returns < 0 for error, socket for ok
  *
  */
-static int create_shortcut(char *atm_addr)
+static int create_shortcut(char *atm_addr, struct atm_qos qos)
 {
         int s, flags, retval;
         struct sockaddr_atmsvc addr;
@@ -638,6 +638,13 @@ static int create_shortcut(char *atm_addr)
         addr.sas_family = AF_ATMSVC;
         memcpy(addr.sas_addr.prv, atm_addr, ATM_ESA_LEN);
 	s = get_socket(&data_listen_addr);
+	if(qos.txtp.traffic_class > ATM_UBR || qos.rxtp.traffic_class > ATM_UBR){
+	        if (setsockopt(s, SOL_ATM,SO_ATMQOS, &qos, sizeof(qos)) < 0){
+		        printf("mpcd: io.c: setsockopt SO_ATMQOS failed: %s \n",strerror(errno));
+			close(s);
+			return -1;
+		}
+	}
         dprintf("mpcd: create_shortcut() got fd %d \n", s);
         if ( (flags = fcntl(s, F_GETFL)) < 0) {
                 printf("mpcd: io.c: fcntl(F_GETFL) failed: %s\n", strerror(errno));
