@@ -389,7 +389,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		  NULL);
 		return;
 	    }
-	    if (!sock || sock->state != ss_connected) {
+	    if (!sock || sock->state != ss_connected || !sock->owner) {
 		send_kernel(sock->id,0,as_okay,-EBADFD,NULL,NULL,NULL,NULL,
 		  NULL);
 		return;
