@@ -367,6 +367,9 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL,&sock->local,
 	      &sock->sap,&sock->qos);
 	    new_state(sock,ss_connected);
+#ifdef Q2963_1
+	    sock->owner = 1;
+#endif
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
 	      &sock->local,0) < 0) strcpy(buffer,"<invalid>");
 	    diag(COMPONENT,DIAG_INFO,"Active open succeeded (CR 0x%06X, "
@@ -384,6 +387,9 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    send_kernel(sock->id,0,as_okay,0,NULL,NULL,&sock->local,&sock->sap,
 	      NULL);
 	    new_state(sock,ss_connected);
+#ifdef Q2963_1
+	    sock->owner = 0;
+#endif
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1, (struct sockaddr *)
 	      &sock->local,0) < 0) strcpy(buffer,"<invalid>");
 	    diag(COMPONENT,DIAG_INFO,"Passive open succeeded (CR 0x%06X, "
@@ -552,7 +558,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
  * Buglet ahead: should actually test "call_state"
  */
 	case ATM_MSG_MODIFY_REQ:
-	    if (sock->state != ss_connected) break;
+	    if (sock->state != ss_connected || sock->owner) break;
 	    sock->new_qos = sock->qos;
 	    if (q_present(&in_dsc,QF_fw_pcr_01))
 		sock->new_qos.rxtp.max_pcr = q_fetch(&in_dsc,QF_fw_pcr_01);
@@ -580,7 +586,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    new_state(sock,ss_mod_fin_fail);
 	    return;
 	case ATM_MSG_CONN_AVAIL:
-	    if (sock->state != ss_connected) break;
+	    if (sock->state != ss_connected || !sock->owner) break;
 	    STOP_TIMER(sock);
 	    send_kernel(sock->id,0,as_modify,ATM_MF_SET,NULL,NULL,NULL,NULL,
 	      &sock->qos);
