@@ -8,6 +8,7 @@
 #include <atm.h>
 #include <atmd.h>
 
+#include "uni.h"
 #include "../fab.h"
 
 
@@ -85,7 +86,7 @@ static int check_ci(struct sockaddr_atmpvc *pvc)
 
 
 void fab_op(CALL *call,int op,const struct atm_qos *qos,
-  void (*callback)(CALL *call,int okay,void *user),void *user)
+  void (*callback)(CALL *call,int cause,void *more,void *user),void *user)
 {
     diag(COMPONENT,DIAG_INFO,"fab_op%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
       !op ? " FREE" : "",op & RM_RSV(_RM_ANY) ? " RM_RSV:" : "",
@@ -101,13 +102,13 @@ void fab_op(CALL *call,int op,const struct atm_qos *qos,
       op & _RM_SHIFT(RM_PATH_RX) ? " PATH_RX" : "");
     if (op & (RM_RSV(RM_IN) | RM_CLAIM(RM_IN)))
 	if (!check_ci(&call->in.pvc)) {
-	    callback(call,0,user);
+	    callback(call,ATM_CV_CI_UNAVAIL,NULL,user);
 	    return;
 	}
     if (op & (RM_RSV(RM_OUT) | RM_CLAIM(RM_OUT)))
 	if (!check_ci(&call->out.pvc)) {
-	    callback(call,0,user);
+	    callback(call,ATM_CV_CI_UNAVAIL,NULL,user);
 	    return;
 	}
-    callback(call,1,user);
+    callback(call,0,NULL,user);
 }
